*==============================================================================
* Program:			FieldsInExpr
* Purpose:			Fills an array with fields in an expression
* Author:			Doug Hennig, mailto:dhennig@stonefield.com
* Copyright:		(c) 2001 Stonefield Systems Group Inc.
* Web site:			http://www.stonefield.com
* Last revision:	02/27/2001
* Parameters:		taFields     - an array (passed by reference using @) to
*						contain the fields
*					tcExpression - the expression
*					tcAlias      - an alias to add to unaliased fields
*						(optional: if it isn't specified, no alias is added)
* Returns:			the number of fields in the array
* Environment in:	none
* Environment out:	none
*==============================================================================

lparameters taFields, ;
	tcExpression, ;
	tcAlias
local lcExpression, ;
	lnFields, ;
	llAddAlias, ;
	lcWords, ;
	lnWords, ;
	lnI, ;
	lcWord, ;
	lnWord, ;
	lcChar, ;
	lcStart

* Define function and field delimiters in expressions.

#define ccDELIMITERS ' !#%^*()-+/,<>[]{}='

* Ensure the necessary parameters were passed.

assert type('taFields[1]') <> 'U' ;
	message 'FieldsInExpr: array not passed'
assert vartype(tcExpression) = 'C' and not empty(tcExpression) ;
	message 'FieldsInExpr: invalid expression passed'

* Set up the variables we need. Because a space before an open paren can fool
* us, let's remove any we find (note that we can't use NORMALIZE() here because
* a field list like COMPANY, CITY is reduced to just COMPANY).

lcExpression = strtran(tcExpression, ' (', '(')
lnFields     = 0
llAddAlias   = vartype(tcAlias) = 'C' and not empty(tcAlias)
lcWords      = ''

* Determine how many "words" there are in the expression, and then check each
* one. Count how many times each word occurs.

lnWords = getwordcount(lcExpression, ccDELIMITERS)
for lnI = 1 to lnWords
	lcWord  = getwordnum(lcExpression, lnI, ccDELIMITERS)
	lnWord  = occurs(lcWord + ',', lcWords) + 1
	lcWords = lcWords + lcWord + ','

* If this isn't a digit, see if it's a function (it's followed by an open
* paren; we need to check this specific occurrence of the word since it may
* appear more than once) or a string. If not, add an alias if necessary and if
* it isn't already in the array, add it.

	if not isdigit(lcWord)
		lcChar  = substr(lcExpression, at(lcWord, lcExpression, lnWord) + ;
			len(lcWord), 1)
		lcStart = left(lcWord, 1)
		if lcChar <> '(' and not lcStart $ ['"] and lcStart <> '['
			if llAddAlias and not '.' $ lcWord
				lcWord = tcAlias + '.' + lcWord
			endif llAddAlias ...
			if ascan(taFields, lcWord) = 0
				lnFields = lnFields + 1
				dimension taFields[lnFields]
				taFields[lnFields] = lcWord
			endif ascan(taFields, lcWord) = 0
		endif lcChar <> '(' ...
	endif not isdigit(lcWord)
next lnI
return lnFields
